# --8<--8<--8<--8<--
#
# Copyright (C) 2007 Smithsonian Astrophysical Observatory
#
# This file is part of App::Env::CIAO
#
# App::Env::CIAO is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# -->8-->8-->8-->8--

package App::Env::CIAO;

use strict;
use warnings;
use Carp;


use Scalar::Util qw( looks_like_number );
use Shell::GetEnv;

our $VERSION = '0.12';

## no critic (ProhibitAccessOfPrivateData)


use constant V43 => 4.3;
use constant V42 => 4.2;
use constant V41 => 4.1;
use constant V40 => 4.0;
use constant V34 => 3.4;

use constant VDEFAULT => 'default';
use constant VLATEST  => V43;

my @Vars = qw( os_type os_vendor os_version );

# if a string, version must be lower cased!
# in order of most to least specific
my @Config = _hash_populate_nested (
                 ########################################
                 [ os_type => 'linux',

                   [ version    => 'x',
		     file => '/proj/xena/Linux_ciaox/bin/ciao.csh',
                   ],

                   [ version    => V34,
                     file => '/soft/ciao3.4/bin/ciao.csh',
                   ],

                   [ version    => V40,
                     file => '/soft/ciao4.0/bin/ciao.csh',
                   ],

                   [ version    => V41,
                     file => '/soft/ciao4.1/bin/ciao.csh',
                   ],

                   [ version    => V42,
                     file => '/soft/ciao4.2/bin/ciao.csh',
                   ],

                   [ version    => V43,
                     file => '/soft/ciao-4.3/bin/ciao.csh',
                   ],

                   [ version    => VDEFAULT,
                     file => '/soft/ciao/bin/ciao.csh',
                   ],

                 ],

                 ########################################
                 [ os_type => 'solaris',

                   [ version    => 'x',
                     file => '/proj/xena/sun10_ciaox/bin/ciao.csh',
                   ],

                   [ version   => V34,
                     file => '/soft/ciao3.4/bin/ciao.csh',
                   ],

                   [ version   => V40,
                     file => '/soft/ciao4.0/bin/ciao.csh',
                   ],

                   [ version   => V41,
                     file => '/soft/ciao4.1/bin/ciao.csh',
                   ],

                   [ version   => V42,
                     file => '/soft/ciao4.2/bin/ciao.csh',
                   ],

                   [ version   => V43,
                     file => '/soft/ciao-4.3/bin/ciao.csh',
                   ],

                   [ version    => VDEFAULT,
                     file => '/soft/ciao/bin/ciao.csh',
                   ],

                 ],

                 ########################################
                 [ os_type => 'darwin',

                   [ hw_cpu_generic => 'x86_64',
                     [
                      version    => 'x',
                      file => '/proj/xena/osxi_ciaox/bin/ciao.csh',
                     ],

                     [ version    => V40,
                       file => '/proj/xena/ciao4/osxi/bin/ciao.csh',
                     ],

                     [ version    => V41,
                       file => '/soft/ciao4.1/bin/ciao.csh',
                     ],

                     [ version    => V42,
                       file => '/soft/ciao4.2/bin/ciao.csh',
                     ],

                     [ version    => V43,
                       file => '/soft/ciao-4.3/bin/ciao.csh',
                     ],

		     [ version    => VDEFAULT,
		       file => '/soft/ciao/bin/ciao.csh',
		     ],

                   ],

                   #
                   [ hw_cpu_generic => 'ppc970',
                     [
                      version    => 'x',
                      file => '/proj/xena/osx_ciaox/bin/ciao.csh',
                     ],

                     [
                      version    => V40,
                      file => '/proj/xena/ciao4/osx/bin/ciao.csh',
                     ],

                     [
                      version    => V34,
                      file => '/proj/xena/ciao_3.4/osx/bin/ciao.csh',
                     ],
                   ]
                 ],

                );



=pod

=begin making_pod_coverage_shut_up

=item envs

=end making_pod_coverage_shut_up

=cut

sub envs
{
    my ( $opt ) = @_;

    my %opt;
    while( my ( $k, $v ) = each %$opt )
    {
	$opt{lc $k} = $v;
    }

    $opt{version} = VDEFAULT unless defined $opt{version};

    $opt{pfiles} = pfiles( $opt{pfiles} ) if defined $opt{pfiles};

    my $setup_file = eval { _get_setup_file( $opt{version} ) };
    die $@ if $@;

    die( "$setup_file does not exist on this system\n" )
      unless -e $setup_file;

    # source the shell script
    # pass Shell::GetEnv's croaks up to caller of App::Env
    my $env = Shell::GetEnv->new( 'tcsh',
				  "source $setup_file",
				{ Startup => 0}
				)->envs;

    if ( defined $opt{pfiles} )
    {
	my ( $op, $where, $dirs )  = @{$opt{pfiles}}{qw[ op where dirs ]};
	## no critic (ProhibitNoStrict)
	no strict 'refs';
	require Config::PFiles::Path;
	my $path = Config::PFiles::Path->new( $env->{PFILES} );
	$path->$op( $where, @$dirs );

	$env->{PFILES} = $path->export;
    }

    return $env;

}

sub pfiles {

    my( $spec ) = @_;

    my ( $op, $where, $dirs ) =
      $spec =~ /^(append|prepend|replace|remove):(rw|ro)(?::(.*))?$/;

    die( "could not parse PFILES specification: $spec\n" )
      unless defined $where;

    my @dirs = defined $dirs ? split( ',', $dirs ) : ();

    return { op => $op, where => $where, dirs => \@dirs };
}

sub _get_setup_file
{
    my ( $version ) = @_;

    require CXC::SysArch;

  CONFIG:
    for my $config ( @Config )
    {
        my @keys = grep { ! /^version$/ && ! /^file$/ } keys %$config;

        next if grep { exists $config->{$_}
                         && $config->{$_} ne $CXC::SysArch::Vars{$_} } @keys;

        if (  exists $config->{version} )
        {
            if ( looks_like_number($version) )
            {
                next CONFIG unless looks_like_number( $config->{version} );
                next CONFIG if $version != $config->{version};
            }
            else
            {
                next CONFIG if lc $version ne $config->{version};
            }
        }

        return $config->{file};
    }

    die( "No CIAO configuration for the requested OS and CIAO version\n" );
}


# create a set of hashes based upon a nested specification scheme,
# where shared attributes occur at upper levels.  For example,
#  [ key1 => val1, [ key2 => val21 ], [ key2 => val22 ] ]
# creates the hashes
#  { key1 => val1, key2 => val21 }, { key1 => val1, key2 => val22 }
#
# attributes are accumulated until an innermost nest (i.e. a level
# which contains only keys and values and no further nested array)

sub _hash_populate_nested
{
    my ( @spec ) = @_;

    my @objs;
    my @stack = map { [ $_, { }, 0 ] } @spec;

  SPEC:
    while ( @stack )
    {
        my ( $spec, $attr, $nested ) = @{ shift @stack };

        while ( @$spec )
        {
            my $key = shift @$spec;

            if ( 'ARRAY' eq ref $key )
            {
                unshift @stack,
                    [ $key, { %$attr }, 0 ],
                    [ $spec, $attr    , 1 ];
                next SPEC;
            }
            else
            {
                die( "missing value for key $key\n" )
                  unless @$spec;
                $attr->{$key} = shift @$spec;
            }
        }
        push @objs, $attr unless $nested;
    }

    return @objs;
}

1;

__END__

=head1 NAME

App::Env::CIAO - retrieve the CIAO environment


=head1 SYNOPSIS

    use App::Env qw( CIAO );


=head1 DESCRIPTION

This module should not be used directly; see B<App::ENV>.  The
following B<AppOpts> options are recognized:

=over

=item version

The release of CIAO for which to retrieve the environment.  It may be one of:

  4.0
  3.4
  x

If no version is specified, the current default version is
used. This module assumes that 4.0 is the current release.

=item pfiles

This specifies an operation to be performed on the C<PFILES>
environment variable after the environment is loaded.  It is
essentially a direct interface to the B<Config::PFiles::Path> module.
The specification should take the form

  op:where:dirs

where I<op> is one of C<append>, C<prepend>, C<replace>, C<remove>,
I<where> is one of C<ro>, C<rw> and I<dirs> is a comma separated list
of directories.  See the documentation for B<Config::PFiles::Path> for
more information.

For example,

   pfiles => 'prepend:rw:.'

will add the current directory to the front of the read-write path.

=back


=head1 INCOMPATIBILITIES

None reported.


=head1 BUGS AND LIMITATIONS

No bugs have been reported.


=head1 LICENSE AND COPYRIGHT

Copyright (c) 2007 The Smithsonian Astrophysical Observatory

App::Env::CIAO is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=head1 AUTHOR

Diab Jerius  E<lt>djerius@cpan.orgE<gt>


